/*
 * main.c
 * 20110422 (c) DE0508, Uwe
 *
 * http://www.nongnu.org/avr-libc/user-manual/mem_sections.html
 */

/*                                    atTiny85
                                    +------------+
        (PCINT5/#RESET/ADC0/dW PB5 -| 1        8 |- VCC
(PCINT3/XTAL1/CLKI/#OC1B/ADC3) PB3 -| 2        7 |- PB2 (SCK/USCK/SCL/ADC1/T0/INT0/PCINT2)
 (PCINT4/XTAL2/CLKO/OC1B/ADC2) PB4 -| 3        6 |- PB1 (MISO/DO/AIN1/OC0B/OC1A/PCINT1)
                               GND -| 4        5 |- PB0 (MOSI/DI/SDA/AIN0/OC0A/#OC1A/AREF/PCINT0)
                                    +------------+

                                      Belegung
                                    +------------+
                        #RESET PB5 -| 1        8 |- VCC
                           KEY PB3 -| 2        7 |- PB2 Relais3
                           LED PB4 -| 3        6 |- PB1 Relais2
                               GND -| 4        5 |- PB0 Relais1
                                    +------------+
*/

/*
 * system includes
 */
#include <avr/io.h>
#include <avr/interrupt.h>

#include <inttypes.h>

/*
 * User Include Files
 */
#include "main.h"
#include "timer.h"
#include "schedule.h"
#include "entprellen.h"

/*
 * Prototypen
 */

/*
 * globale Variable
 */


/*
 * user functions
 */
enum ps_state_t {OFF, ON};

#define bit_ON(bit)		sbit(PRR,bit)
#define bit_OFF(bit)		cbit(PRR,bit)

#ifdef PRR
# define POWERSAVE_USI(_state)		bit_##_state(PRUSI)
# define POWERSAVE_ADC(_state)		bit_##_state(PRADC)
# define POWERSAVE_TIMER1(_state)	bit_##_state(PRTIM1)

#else
# define POWERSAVE_USI(_state)
# define POWERSAVE_ADC(_state)
# define POWERSAVE_TIMER1(_state)
#endif

static void cpu_power_save_init(void)
{
  POWERSAVE_USI(ON);

  // Analog Comparator abschalten
  cbit(ACSR, ACIE); sbit(ACSR, ACD);

  // adc abschalten
  cbit(ADCSRA, ADEN);
  POWERSAVE_ADC(ON);
  POWERSAVE_TIMER1(ON);
}


static void system_init(void)
{
  // Input- und Outputports initialisieren
  SET_OUTPUT(RELAIS1);
  SET_OFF(RELAIS1);					// high aktiv

  SET_OUTPUT(RELAIS2);
  SET_OFF(RELAIS2);					// high aktiv

  SET_OUTPUT(RELAIS3);
  SET_OFF(RELAIS3);					// high aktiv

#ifdef USE_TASK_LED
  SET_OUTPUT(LED);
  SET_ON(LED);						// low aktiv
#endif

#ifdef USE_TASK_KEY
  SET_INPUT(KEY);
  SET_PULLUP_ON(KEY);					// low aktiv
#endif

  // alle Zähler initialisieren
  Setup_Timer0( T0_COUNTER, _T0_PRESCALE_256 );

  /* Initialize Scheduler */
  timerinit();
}


/* 
 * TASK Prototypen 
 */
static void task_relais1(void);
static void task_relais2(void);
static void task_relais3(void);

#ifdef USE_TASK_LED
static void task_led(void);
#endif

#ifdef USE_TASK_KEY
static void task_key(void);
#endif


static void task_relais1(void)
{
  // Schaltet Relais1 ein
  SET_ON(RELAIS1);
}

static void task_relais2(void)
{
  // Schaltet Relais2 ein
  SET_ON(RELAIS2);
}

static void task_relais3(void)
{
  // Schaltet Relais3 ein
  SET_ON(RELAIS3);
}


#ifdef USE_TASK_LED
static void task_led(void)
{
  // die LED blinkt alle 1 Sek und zeigt den Betriebszustand an.
  SET_TOGGLE(LED);
  ADD_TASK_LED();
}
#endif


#ifdef USE_TASK_KEY
static void task_key(void)
{
  // Wenn die Taste gedrückt wurde, 
  // dann schalte alle Relais aus und starte alle Timer neu!
  if ( get_key_press(_BV(KEY_BIT)) ) {

    // alle Relais aus
    SET_OFF(RELAIS1);				// high aktiv
    SET_OFF(RELAIS2);				// high aktiv
    SET_OFF(RELAIS3);				// high aktiv

    // startet alle Timer neu !
    REM_TASK_RELAIS1();
    REM_TASK_RELAIS2();
    REM_TASK_RELAIS3();

    ADD_TASK_RELAIS1();
    ADD_TASK_RELAIS2();
    ADD_TASK_RELAIS3();
  }
  ADD_TASK_KEY();
}
#endif


/*
 * main - das Hauptprogramm
 */
extern int main (void) __attribute__((OS_main));
int main(void)
{
  /* Setup ports, clock... */
  system_init();

  cpu_power_save_init();

  /* Interrupt on */
  sei();

  ADD_TASK_RELAIS1();
  ADD_TASK_RELAIS2();
  ADD_TASK_RELAIS3();

#ifdef USE_TASK_LED
  ADD_TASK_LED();
#endif

#ifdef USE_TASK_KEY
  ADD_TASK_KEY();
#endif

  /* Hauptprogrammschleife */
  for(;;)
  {
    if ( f_timer_tick ) { 	// 1ms vergangen
      f_timer_tick = 0;
      timertick();
    }
  }

  return 0;
}

